# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - MoinMessageSupport library

    @copyright: 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.Page import Page
from MoinMoin.log import getLogger
from MoinMoin.user import User
from MoinMoin import wikiutil
from MoinSupport import ItemStore, getHeader, getMetadata, getWikiDict, \
                        writeHeaders
from MoinMessage import GPG, Message, MoinMessageError, \
                        is_signed, is_encrypted, getContentAndSignature
from email.parser import Parser
import time

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

Dependencies = ['pages']

class MoinMessageAction:

    "Common message handling support for actions."

    def __init__(self, pagename, request):

        """
        On the page with the given 'pagename', use the given 'request' when
        reading posted messages, modifying the Wiki.
        """

        self.pagename = pagename
        self.request = request
        self.page = Page(request, pagename)
        self.store = ItemStore(self.page, "messages", "message-locks")

    def do_action(self):
        request = self.request
        content_length = getHeader(request, "Content-Length", "HTTP")
        if content_length:
            content_length = int(content_length)

        self.handle_message_text(request.read(content_length))

    def handle_message_text(self, message_text):

        "Handle the given 'message_text'."

        message = Parser().parse(StringIO(message_text))
        self.handle_message(message)

    def handle_message(self, message):

        "Handle the given 'message'."

        # Detect PGP/GPG-encoded payloads.
        # See: http://tools.ietf.org/html/rfc3156

        if is_signed(message):
            self.handle_signed_message(message)
        elif is_encrypted(message):
            self.handle_encrypted_message(message)

        # Reject unsigned and unencrypted payloads.

        else:
            request = self.request
            writeHeaders(request, "text/plain", getMetadata(self.page), "415 Unsupported Media Type")
            request.write("Only PGP/GPG-signed payloads are supported.")

    def handle_encrypted_message(self, message):

        "Handle the given encrypted 'message'."

        request = self.request

        homedir = self.get_homedir()
        if not homedir:
            return

        gpg = GPG(homedir)

        try:
            text = gpg.decryptMessage(message)

        # Reject messages without a declaration.

        except MoinMessageMissingPart:
            writeHeaders(request, "text/plain", getMetadata(self.page), "415 Unsupported Media Type")
            request.write("There must be a declaration and a content part for encrypted uploads.")
            return

        # Reject messages without appropriate content.

        except MoinMessageBadContent:
            writeHeaders(request, "text/plain", getMetadata(self.page), "415 Unsupported Media Type")
            request.write("Encrypted data must be provided as application/octet-stream.")
            return

        # Reject any unencryptable message.

        except MoinMessageError:
            writeHeaders(request, "text/plain", getMetadata(self.page), "403 Forbidden")
            request.write("The message could not be decrypted.")
            return

        # Log non-fatal errors.

        if gpg.errors:
            getLogger(__name__).warning(gpg.errors)

        # Handle the embedded message which may itself be a signed message.

        self.handle_message_text(text)

    def handle_signed_message(self, message):

        "Handle the given signed 'message'."

        request = self.request

        # Accept any message whose sender was authenticated by the PGP method.

        if request.user and request.user.valid and request.user.auth_method == "pgp":

            # Handle the embedded message.

            content, signature = getContentAndSignature(message)
            self.handle_message_content(content)

        # Reject any unverified message.

        else:
            writeHeaders(request, "text/plain", getMetadata(self.page), "403 Forbidden")
            request.write("The message could not be verified. "
                          "Maybe this site is not performing authentication using PGP signatures.")

    def handle_message_content(self, content):

        "Handle the given message 'content'."

        request = self.request

        # Interpret the content as one or more updates.

        message = Message()
        message.handle_message(content)

        # Test any date against the page or message store.

        if message.date:
            store_date = time.gmtime(self.store.mtime())
            page_date = time.gmtime(wikiutil.version2timestamp(self.page.mtime_usecs()))
            last_date = max(store_date, page_date)

            # Reject messages older than the page date.

            if message.date < last_date:
                writeHeaders(request, "text/plain", getMetadata(self.page), "403 Forbidden")
                request.write("The message is too old: %s versus %s." % (message.date, last_date))
                return

        # Reject messages without dates if so configured.

        elif getattr(request.cfg, "moinmessage_reject_messages_without_dates", True):
            writeHeaders(request, "text/plain", getMetadata(self.page), "403 Forbidden")
            request.write("The message has no date information.")
            return

        # Handle the message as an object.

        self.handle_message_object(message)

    def get_homedir(self):

        "Locate the GPG home directory."

        request = self.request
        homedir = get_homedir(self.request)

        if not homedir:
            writeHeaders(request, "text/plain", getMetadata(self.page), "415 Unsupported Media Type")
            request.write("Encoded data cannot currently be understood. Please notify the site administrator.")

        return homedir

def get_homedir(request):

    "Locate the GPG home directory."

    return getattr(request.cfg, "moinmessage_gpg_homedir")

# vim: tabstop=4 expandtab shiftwidth=4
